//---------------------------------------------------------------------
// Ehrenfest Urn Model
// The Ehrenfest urn model is a model for diffusion of gasses. 
// Assume two urns and N balls. 
// The Markov states are given by the number of balls in the first urn, 
// i.e. we have N+1 states s_i, i=0,...,N. 
// (s_0: all balls in second urn, s_N: all balls in first urn). 
// At each step, one of the N balls is chosen by random 
// and moved to the other urn. 
// 
// c) Implement a Markov Chain Monte Carlo simulation for this model, 
//    and determine the equilibrium distribution for N = 4,8,16
//---------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>

void Tmatrix_MulEqul(double** M2, double **M, int N){
  double **T1 = new double*[N+1];
  double **T2 = new double*[N+1];
  for(int i = 0 ; i < N+1 ; i++){
    T1[i] = new double[N+1];
    T2[i] = new double[N+1];
    for(int j = 0 ; j < N+1 ; j++){
      T1[i][j] = 0.0;
      T2[i][j] = 0.0;
    }
  }

  //T1=M2
  for(int i = 0 ; i < N+1 ; i++){
    for(int j = 0 ; j < N+1 ; j++){
      T1[i][j] = M2[i][j];
    }
  }

  //T2=0
  for(int i = 0 ; i < N+1 ; i++){
    for(int j = 0 ; j < N+1 ; j++){
      T2[i][j] = 0.0;
    }
  }

  //T2=T1*M
  for(int i = 0 ; i < N+1 ; i++){
    for(int j = 0 ; j < N+1 ; j++){
      for(int k = 0 ; k < N+1 ; k++){
        T2[i][j] += T1[i][k]*M[k][j];
      }
    }
  }

  //M=T1
  for(int i = 0 ; i < N+1 ; i++){
    for(int j = 0 ; j < N+1 ; j++){
      M2[i][j] = T2[i][j];
    }
  }


  for(int i = 0 ; i < N+1 ; i++){
    delete[] T1[i];
    delete[] T2[i];
  }
  delete[] T1;
  delete[] T2;

}

int irand(int N){
  int ran = rand()%N;
  return ran;
}

void transition(int *Z, int state[], int N){

  if( irand(N) < state[0] ){ // select left ball
    // move the ball from left to right
    state[0]--;
    state[1]++;
  }
  else{ //select right ball
    // move the ball from right to left
    state[0]++;
    state[1]--;
  }

  Z[state[0]]++;

}


int main(int argc, char* argv[]){

  if(argc != 2){
    printf("input N(=number of balls)\n");
    exit(0);
  }

  int N = atoi(argv[1]); // the number of balls
  int N_Thermal = 10000;
  int N_Measure = 100000;

  //------------------------
  // analytic calculation
  // using transition matrix
  //------------------------
  double **M = new double*[N+1];
  double **M2 = new double*[N+1];
  for(int i = 0 ; i < N+1 ; i++){
    M[i] = new double[N+1];
    M2[i] = new double[N+1];
    for(int j = 0 ; j < N+1 ; j++){
      M[i][j] = 0.0;
      M2[i][j] = 0.0;
    }
  }
  
  for(int i = 0 ; i < N ; i++){
    int j=i+1;
    M[i][j] = 1.0 - (double)i/N;
    M2[i][j] = 1.0 - (double)i/N;
  }
  for(int i = 1; i < N+1 ; i++){
    int j=i-1;
    M[i][j] = (double)i/N;
    M2[i][j] = (double)i/N;
  }
 
  double *Z_anal = new double[N+1];
  for(int i = 0 ; i < N+1 ; i++){
    Z_anal[i] = 0.0 ;
  }

  FILE *fp_anal;
  char fname_anal[1024];
  sprintf(fname_anal,"anal_N%d.dat",N);
  fp_anal = fopen(fname_anal, "w");

  //corresponding to thermalization
  for(int i = 0 ; i < N_Thermal-1 ; i++){
    Tmatrix_MulEqul(M2, M, N);
  }

  // compute analyically
  for(int i = 0; i < N_Measure ; i++){
    Tmatrix_MulEqul(M2, M, N);
    for(int j = 0 ; j < N+1 ; j++){
      Z_anal[j] += M2[0][j];
    }
  }

  for(int i = 0 ; i < N+1 ; i++){
    fprintf(fp_anal,"%d %lf\n", i, Z_anal[i]);
  }

  //----------------
  // MCMC simulation
  //----------------
  int state[2] = {0, N};

  int *Z = new int[N+1];
  for(int i = 0 ; i < N+1 ; i++){
    Z[i] = 0 ;
  }

  FILE *fp_dat;
  char fname_dat[1024];
  sprintf(fname_dat,"N%d.dat",N);
  fp_dat = fopen(fname_dat, "w");

  // Thermalize
  for(int i = 0 ; i < N_Thermal ; i++){
    transition(Z, state, N);
  }
  // initiate the measurement variable
  for(int i = 0 ; i < N+1 ; i++){
    Z[i] = 0 ;
  }

  // measure
  for(int i = 0 ; i < N_Measure ; i++){
    transition(Z, state, N);
  }

  for(int i = 0 ; i < N+1 ; i++){
    fprintf(fp_dat,"%d %d\n", i, Z[i]);
  }

  delete[] Z;

  for(int i = 0 ; i < N+1 ; i++){
    delete[] M[i];
  }
  delete[] M;


  return 0;
}
